/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <cairo.h>
#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <math.h>
#include "edecor.h"
#include "action.h"
#include "calibration.h"
#include "misc.h"
#include "profile.h"
#include "protocol.h"
#include "unlock.h"
#include "clicker.h"

gboolean showing_clicker	= FALSE;

static GtkWidget* window;

static const guint	width [N_CLICKER_SIZES]			= { 50, 100, 200 };
static const guint	height [N_CLICKER_SIZES]		= { 50, 100, 200 };
static const guint	radius_inner [N_CLICKER_SIZES]	= { 7, 14, 28 };
static const guint	radius_outer [N_CLICKER_SIZES]	= { 18, 36, 72 };

static gint size = 0;
static gint step = 0;

static gboolean
window_expose_cb	(GtkWidget *widget, GdkEventExpose *event, gpointer data);


	void
create_clicker (void)
{
	/* Create the window */
	window = gtk_window_new (GTK_WINDOW_POPUP);

	g_signal_connect (window,
			"expose-event", G_CALLBACK (window_expose_cb), NULL);

	gtk_window_set_title (GTK_WINDOW (window), "enPathia");
	gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
	gtk_widget_set_app_paintable (window, TRUE);
	gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);		

#ifndef G_OS_WIN32
	GdkScreen*		screen;
	GdkColormap*	colormap;

	/* Set RGBA colormap */ 
	screen = gtk_widget_get_screen (window);
	colormap = gdk_screen_get_rgba_colormap (screen);

	gtk_widget_set_colormap (window, colormap);
#endif
}

	void
show_clicker (gint x, gint y)
{
	if (showing_clicker || calibrating || unlocking)
		return;

	showing_clicker = TRUE;
	step = 0;

	/* Change to notification mode */
	change_mode (MODE_NOTIFY);

	/* Center the window where we are asked to and show it */
	x -= (width [size] >> 1);
	if (x < 0)
		x = 0;
	else if (x + width [size] > screen_width)
		x = screen_width - width [size];

	y -= (height [size] >> 1);
	if (y < 0)
		y = 0;
	else if (y + height [size] > screen_height)
		y = screen_height - height [size];

	gtk_window_move (GTK_WINDOW (window), x, y);
	gtk_window_present (GTK_WINDOW (window));

#ifdef __MACH__
	gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);		
#endif
}

	void
hide_clicker (void)
{
	showing_clicker = FALSE;

	gtk_widget_hide (window);
}

	void
set_clicker_size (gint clicker_size)
{
	gint	i;

	if (clicker_size < 0)
		clicker_size = 0;
	else if (clicker_size > N_CLICKER_SIZES - 1)
		clicker_size = N_CLICKER_SIZES - 1;

	size = clicker_size;

	gtk_window_resize (GTK_WINDOW (window), width [size], height [size]);

#ifdef G_OS_WIN32
	GdkDrawable*	mask;
	cairo_t*		cr;

	/* Draw the mask */
	mask = (GdkDrawable*) gdk_pixmap_new (NULL, width [size], height [size], 1);
	cr = gdk_cairo_create (mask);

	cairo_set_source_rgb (cr, 0, 0, 0);
	cairo_rectangle (cr, 0, 0, width [size], height [size]);
	cairo_fill (cr);

	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
	cairo_set_source_rgb (cr, 1, 1, 1);
	cairo_arc (cr, width [size] >> 1, height [size] >> 1, radius_outer [size], 0, 2.0 * M_PI);
	cairo_fill (cr);

	cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
	cairo_set_source_rgb (cr, 0, 0, 0);
	cairo_arc (cr, width [size] >> 1, height [size] >> 1, radius_inner [size], 0, 2.0 * M_PI);
	cairo_fill (cr);

	cairo_destroy (cr);

	/* And apply it */
	gtk_widget_shape_combine_mask (window, NULL, 0, 0);
	gtk_widget_shape_combine_mask (window, mask, 0, 0);

	g_object_unref (mask);
#endif
}

	void
clicker_handle_movement (gchar movement)
{
	hide_clicker ();

	change_mode (MODE_MOVEMENT);
}

	void
clicker_handle_stopped (void)
{
	step++;

	if (step > CLICKER_N_STEPS)
	{
		hide_clicker ();

#ifdef G_OS_UNIX
		/* Wait for the window under the pointer to grab the focus */
		wait_ms (100);
#endif

		execute_action (CLICK);
	}
	else
		gtk_widget_queue_draw (window);
}

	static gboolean
window_expose_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t*			cr;
	cairo_pattern_t*	pat;
	gint				xc, yc;	
	gint 				i;
	gdouble				angle;

	xc = width [size] >> 1;
	yc = height [size] >> 1;

	cr = gdk_cairo_create (widget->window);

#ifdef G_OS_WIN32
	cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);

	/* Background */
	cairo_set_source_rgb (cr, 1, 1, 1);
	cairo_rectangle (cr, 0, 0, width [size], height [size]);
	cairo_fill (cr);
#else
	/* Set operator SOURCE to replace destination */
	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);

	/* Transparent background */
	cairo_set_source_rgba (cr, 0, 0, 0, 0);
	cairo_paint (cr);

	/* Back to OVER operator */
	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
#endif

	/* Bar */	
	pat = cairo_pattern_create_radial (
			xc, yc, radius_inner [size],
			xc, yc, radius_outer [size]);

	cairo_pattern_add_color_stop_rgb (pat, 0.0, 0.53, 0.79, 0.92);
	cairo_pattern_add_color_stop_rgb (pat, 0.5, 0.20, 0.59, 0.74);
	cairo_pattern_add_color_stop_rgb (pat, 1.0, 0.53, 0.79, 0.92);

	cairo_set_source (cr, pat);
	cairo_arc (cr, xc, yc, radius_outer [size], 0, 2.0 * M_PI * step / CLICKER_N_STEPS);
	cairo_line_to (cr, xc, yc);
	cairo_fill (cr);

	cairo_pattern_destroy (pat);

#ifndef G_OS_WIN32
	/* Clear the center */
	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
	cairo_set_source_rgba (cr, 0, 0, 0, 0);

	cairo_arc (cr, xc, yc, radius_inner [size], 0, 2.0 * M_PI);
	cairo_fill (cr);

	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
#endif

	/* Step lines */
#ifdef G_OS_WIN32
	cairo_set_line_width (cr, 1.0);
	cairo_set_source_rgb (cr, 1, 1, 1);
#else
	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
	cairo_set_source_rgba (cr, 0, 0, 0, 0);
#endif

	for (i = 0; i < step; i++)
	{
		angle = 2.0 * M_PI * i / CLICKER_N_STEPS;

		cairo_move_to (cr, xc, yc);
		cairo_line_to (cr,
				xc + radius_outer [size] * cos (angle),
				yc + radius_outer [size] * sin (angle));
		cairo_stroke (cr);
	}

	/* Border */
	cairo_set_line_width (cr, 2.0);
	cairo_set_source_rgb (cr, 0.4, 0.4, 0.4);
	
	cairo_arc (cr, xc, yc, radius_outer [size], 0, 2.0 * M_PI);
	cairo_stroke (cr);

	cairo_arc (cr, xc, yc, radius_inner [size], 0, 2.0 * M_PI);
	cairo_stroke (cr);

	cairo_destroy (cr);

	return FALSE;
}
